/*
 * Pan - A Newsreader for X
 * Copyright (C) 1999  Pan Development Team (pan@superpimp.org)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#include <config.h>

#include <string.h>
#include "pan.h"

article_data*
article_new (void)
{
	article_data* adata = g_new0 (article_data, 1);
	adata->threads = NULL;
	return adata;
}

void
article_add_flag (const server_data *sdata,
		  const group_data *gdata,
		  article_data* adata,
		  int flag,
		  gboolean sync)
{
	adata->state |= flag;
	article_update (sdata, gdata, adata, sync);

	if (flag==STATE_READ)
		articlelist_adata_read_changed (adata);
}

void
article_remove_flag (const server_data *sdata,
		     const group_data *gdata,
		     article_data* adata,
		     int flag,
		     gboolean sync)
{
	adata->state &= ~flag;
	article_update (sdata, gdata, adata, sync);
	articlelist_update_node (adata);

	if (flag==STATE_READ)
		articlelist_adata_read_changed (adata);
}

void
article_update (const server_data *sdata,
		const group_data *gdata,
		const article_data* adata,
		gboolean sync)
{
	article_db db = NULL;

	/* entry assertions */
	g_assert (adata && adata->message_id);

	/* update the database... */
	db = ahdb_ref (sdata, gdata);
	ahdb_save (db, adata, sync);
	ahdb_unref (db);

	/* update the article ui... */
	articlelist_update_node (adata);
}

article_data*
article_copy (article_data* target,
	      const article_data* source)
{
	target->number = g_strdup (source->number);
	target->subject = g_strdup (source->subject);
	target->author = g_strdup (source->author);
	target->message_id = g_strdup (source->message_id);
	target->references = g_strdup (source->references);
	target->date = source->date;
	target->linecount = source->linecount;
	target->parts = source->parts;
	target->part = source->part;
	target->followup_to = g_strdup (source->followup_to);
	target->reply_to = g_strdup (source->reply_to);
	target->newsgroups = g_strdup (source->newsgroups);
	target->state = source->state;

	return target;
}

void
article_free_gfunc (gpointer data, gpointer user_data)
{
	article_free ( (article_data*)data );
}

void
article_free (article_data *adata)
{
        g_return_if_fail (adata!=NULL);

	g_assert (adata->subject != NULL);
	g_free (adata->number);
	g_free (adata->subject);
	adata->subject = NULL;
	g_free (adata->author);
	g_free (adata->message_id);
	g_free (adata->references);
	g_free (adata->newsgroups);
	g_free (adata->reply_to);
	g_free (adata->followup_to);
	g_free (adata);
}



/*
 * Extracts the requested header from adata->header;
 * hdr is the case sensitive name, without the colon
 * Doesn't handle wrapped lines
 */
#if 0
gchar *
extract_header (article_data *adata, gchar *hdr)
{
	gchar* header = g_strconcat (hdr, ": ", NULL);
//	gchar* start = strstr (adata->header, header);
        gchar* end;
	g_free (header);
	if (start == NULL)
		return NULL;
	else {
		start = strchr (start, ':');
		end = strchr (start, '\n');
		return g_strndup (start + 2, end - start - 3);
	}
}
#endif

