/*
 * Pan - A Newsreader for X
 * Copyright (C) 1999  Pan Development Team (pan@superpimp.org)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#include <config.h>

#include <ctype.h>

#include "acache.h"
#include "gui.h"
#include "grouplist.h"
#include "pan.h"
#include "queue.h"
#include "queue-item-body.h"
#include "text.h"

static GtkWidget *text_rightclick_popup;


/***
 *** global variables
 ***/
GtkWidget *text = NULL;

gboolean rot13_enabled = FALSE;
gboolean show_all_headers_in_body = FALSE;

static GnomeUIInfo text_menu_popup[] = {
	{ GNOME_APP_UI_ITEM,
		N_("Open in Window"),
		N_("Open this message for reading."),
		message_read_window },
	GNOMEUIINFO_SEPARATOR,
	{ GNOME_APP_UI_ITEM,
		N_("_Followup to newsgroup"),
		N_("Post a reply to the message on the news server."),
		message_followup_window },
	{ GNOME_APP_UI_ITEM,
		N_("Reply by _E-mail"),
		N_("Create a mail reply to the sender."),
		message_reply_window },
#if 0
	{ GNOME_APP_UI_ITEM,
		N_("Forward"),
		N_("Forward this message to someone."),
		not_implemented },
#endif
	GNOMEUIINFO_SEPARATOR,
#if 0
	{ GNOME_APP_UI_ITEM,
		N_("Show Full Header"),
		N_("Show the full header for the current message."),
		not_implemented },
	GNOMEUIINFO_SEPARATOR,
#endif
	{ GNOME_APP_UI_ITEM,
		N_("Select All"),
		N_("Select all messages"),
		text_select_all,
		NULL, NULL, GNOME_APP_PIXMAP_NONE, NULL,
		'A', GDK_CONTROL_MASK, NULL},

	GNOMEUIINFO_END
};

static gchar welcome[] =
{
"Pan " VERSION "\n"
"Copyright (C) 1999-2000  Pan Development Team (pan@superpimp.org) \n"
"\n"
"Pan homepage: http://www.superpimp.org/ \n"
"\n"
"This program is free software; you can redistribute it and/or modify \n"
"it under the terms of the GNU General Public License as published by \n"
"the Free Software Foundation; either version 2 of the License, or \n"
"(at your option) any later version. \n"
"\n"
"This program is distributed in the hope that it will be useful, \n"
"but WITHOUT ANY WARRANTY; without even the implied warranty of \n"
"MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the \n"
"GNU General Public License for more details. \n"
"\n"
"You should have received a copy of the GNU General Public License \n"
"along with this program; if not, write to the Free Software \n"
"Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA"
};


/***
 *** Private Function Prototypes
 ***/
static void realize_text_cb (GtkWidget *text, gpointer data);
static void text_button_press (GtkWidget *widget, GdkEventButton *bevent);


/***
 *** Public Routines
 ***/

void
text_select_all (void)
{
	pan_lock();
	gtk_editable_select_region (GTK_EDITABLE(text), 0, -1);
	pan_unlock();
}

void
text_rot13_update (void)
{
	/*FIXME: this limits rot13'ing to the main window.
	  FIXME: this ties text to the articlelist for server/group. */
	if (Pan.active_message_id)
	{
		text_set_from_message_id (
			articlelist_get_current_server(),
			articlelist_get_current_group(),
			Pan.active_message_id,
			NNTP_ARTICLE_READ,
			FALSE );
	}
}

/*--------------------------------------------------------------------
 * called when the widget is created and then first drawn, puts
 * our welcome text into the messag pane.
 *--------------------------------------------------------------------*/
static void
realize_text_cb (GtkWidget *text, gpointer data)
{
	GtkAdjustment* hadj = GTK_ADJUSTMENT(GTK_TEXT(text)->hadj);
	GtkAdjustment* vadj = GTK_ADJUSTMENT(GTK_TEXT(text)->vadj);

	gui_set_headers_default (NULL, NULL);
	gtk_editable_delete_text (GTK_EDITABLE(text), 0, -1);

	gtk_text_insert (GTK_TEXT(text), NULL, &text->style->black, NULL, welcome, -1);
	gtk_text_set_point (GTK_TEXT(text), 0);

	gtk_adjustment_set_value (GTK_ADJUSTMENT(vadj), 0);
	gtk_text_set_adjustments (GTK_TEXT(text), hadj, vadj);

	gtk_signal_disconnect_by_func (GTK_OBJECT(text), GTK_SIGNAL_FUNC (realize_text_cb), NULL);
}


/*---[ text_button_press ]--------------------------------------------
 * callback for when a mouse button is pressed inside the text widget
 *--------------------------------------------------------------------*/
static void
text_button_press (GtkWidget *widget, GdkEventButton *bevent)
{
	gtk_widget_grab_focus (GTK_WIDGET (text));

	if (bevent->button != 1)
		gtk_signal_emit_stop_by_name (GTK_OBJECT(widget), "button_press_event");
	
	if (bevent->button == 3)
	{
		const gboolean sel = current_article != NULL;

		GList* l = GTK_MENU_SHELL(text_rightclick_popup)->children; /* l is "open in window" */
		gtk_widget_set_sensitive ( GTK_WIDGET(l->data), sel );
		l = l->next; /* l is separator */
		l = l->next; /* l is "followup to group */
		gtk_widget_set_sensitive ( GTK_WIDGET(l->data), sel );
		l = l->next; /* l is "reply by e-mail" */
		gtk_widget_set_sensitive ( GTK_WIDGET(l->data), sel );
//		l = l->next; /* l is separator */
//		l = l->next; /* l is "select all" */
//		gtk_widget_set_sensitive ( GTK_WIDGET(l->data), sel );

//		gtk_menu_popup (GTK_MENU(text_rightclick_popup), NULL, NULL, NULL,
//				NULL, bevent->button, bevent->time);
		gnome_popup_menu_do_popup (text_rightclick_popup, NULL,
					   NULL, bevent, NULL);
	}
}


/*---[ text_create ]--------------------------------------------------
 * create a scrolled text area that displays a message's body
 *--------------------------------------------------------------------*/
GtkWidget *
text_create (void)
{
	GtkWidget *scrolled_window;

	text = gtk_text_new (NULL, NULL);

	show_all_headers_in_body = gnome_config_get_bool(
		"Pan/Display/Show_All_Headers_In_Body=false");

	gtk_text_set_editable (GTK_TEXT(text), FALSE);
	widget_set_font (GTK_WIDGET(text), message_body_font);
	/* gtk_text_set_word_wrap (GTK_TEXT(text), TRUE); */

	scrolled_window = gtk_scrolled_window_new (NULL, NULL);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
					GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

	/* Add a handler to put a message in the text widget when it is realized */
	gtk_signal_connect (GTK_OBJECT(text), "realize",
			    GTK_SIGNAL_FUNC (realize_text_cb), NULL);

	gtk_signal_connect (GTK_OBJECT(text), "button_press_event",
			    GTK_SIGNAL_FUNC (text_button_press), NULL);

	gtk_container_add (GTK_CONTAINER (scrolled_window), text);

	text_rightclick_popup = gnome_popup_menu_new (text_menu_popup);

	return scrolled_window;
}

/**
***
**/

void
text_update (Message *msg)
{
	gchar *body;
	article_data *adata = NULL;
	article_db db;

        /* sanity checks */
        g_return_if_fail (msg!=NULL);
        g_return_if_fail (msg->message_id!=NULL);
        g_return_if_fail (*msg->message_id!='\0');

        /* update active_message_id */
	g_free (Pan.active_message_id);
	Pan.active_message_id = g_strdup (msg->message_id);
	
        /* load article header */
	db = ahdb_ref (msg->sdata, msg->gdata);
	adata = ahdb_get (db, msg->message_id);
	ahdb_unref (db);

        /* load article body */
	body = acache_load_body (msg->message_id);

	if (body)
	{
		if (rot13_enabled)
		{
			const gchar a [] = "abcdefghijklmnopqrstuvwxyz";
			gchar *pch = NULL;
			for (pch=body; *pch!='\0'; ++pch)
			{
				if (isalpha((int)*pch))
				{
					const gboolean uc = isupper((int)*pch);
					*pch = tolower(*pch);
					*pch = a[((*pch-'a')+13)%26];
					if (uc)
						*pch = toupper(*pch);
				}
			}
		}

		if (use_message_window_always || (msg->needs_window))
		{
			Message* m2 = message_copy (msg);
			message_import_adata (m2, adata);
			message_window_new (m2);
		}
		else
		{
		        /* update the text window */
			update_body_pane (text, body);
			gui_set_headers_default (NULL, adata);

                        /* update current article */
                        if (current_article)
			        article_free (current_article);
			current_article = article_copy (article_new(), adata);
		}
	}

	g_free (body);

	message_free (msg);
	article_free (adata);
}


void
text_set_from_message_id (
	const server_data *sdata,
	group_data *gdata,
	const gchar *message_id,
	MessageWindowType type,
	gboolean needs_window)
{
	Message *msg = NULL;

	/* sanity checks */
	g_return_if_fail (sdata!=NULL);
	g_return_if_fail (gdata!=NULL);
	g_return_if_fail (message_id!=NULL);
	g_return_if_fail (*message_id!='\0');

	/* create message object */
	msg = message_new (sdata, gdata, NULL);
	msg->message_id = g_strdup (message_id);
	msg->type = type;
	msg->needs_window = needs_window;

	if ( acache_body_exists(message_id) )
		text_update (msg);
	else {
		queue_add (QUEUE_ITEM(
			queue_item_body_new(
				grouplist_get_current_server(),
				articlelist_get_current_group(), msg)));
	}
}
#if 0
void text_page_down (void)
{
        GtkText* t = GTK_TEXT(text);
        gfloat upper = t->vadj->upper - text->vadj->page_size;
        t->vadj->value -= t->vadj->page_increment;
        t->vadj->value = MIN (t->vadj->value, upper);
        t->vadj->value = MAX (t->vadj->value, 0.0);
        gtk_signal_emit_by_name (GTK_OBJECT (t->vadj), "value_changed");
}

void text_page_down (void)
{
        GtkText* t = GTK_TEXT(text);
        gfloat upper = t->vadj->upper + text->vadj->page_size;
        t->vadj->value -= t->vadj->page_increment;
        t->vadj->value = MIN (t->vadj->value, upper);
        t->vadj->value = MAX (t->vadj->value, 0.0);
        gtk_signal_emit_by_name (GTK_OBJECT (t->vadj), "value_changed");
}
#endif

